;/*---------------------------------------------------------------------------------------------------------*/
;/*                                                                                                         */
;/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
;/*                                                                                                         */
;/*---------------------------------------------------------------------------------------------------------*/

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Stack_Size      EQU     0x00000400
                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000100	 ; 1536 bytes for heap

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset
                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                                                  ; maximum of 32 External Interrupts are possible
                DCD     BOD_IRQHandler            ; Brownout low voltage detector interrupt
                DCD     WDT_IRQHandler            ; Watch Dog Timer interrupt
                DCD     EINT0_IRQHandler          ; External signal interrupt from PB.0 pin
                DCD     EINT1_IRQHandler          ; External signal interrupt from PB.1 pin
                DCD     GPAB_IRQHandler           ; External signal interrupt from PA[15:0] / PB[7:2]
                DCD     ALC_IRQHandler            ; Automatic Level Control Interrupt
                DCD     PWMA_IRQHandler           ; PWM0 , PWM1 interrupt
                DCD     0                         ; Reserved
                DCD     TMR0_IRQHandler           ; Timer 0 interrupt
                DCD     TMR1_IRQHandler           ; Timer 1 interrupt
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     UART02_IRQHandler         ; UART0 interrupt
                DCD     0                         ; Reserved
                DCD     SPI0_IRQHandler           ; SPI0 interrupt
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     I2C0_IRQHandler           ; I2C0 interrupt
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     TALARM_IRQHandler         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     ACMP_IRQHandler           ; Analog Comparator-0 or Comaprator-1 interrupt
                DCD     PDMA_IRQHandler           ; PDMA interrupt
                DCD     I2S_IRQHandler            ; I2S interrupt
                DCD     CAPS_IRQHandler           ; Capacitive Touch Sensing Relaxation Oscillator Interrupt
                DCD     SDADC_IRQHandler          ; ADC interrupt
                DCD     Default_Handler           ; Reserved
                DCD     RTC_IRQHandler            ; Real time clock interrupt         
                
                                
                
                
                
                AREA    |.text|, CODE, READONLY
                
                					
                
; Reset Handler 
                
                ENTRY
                
Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  __main
                
				;check ICE_CLK and ICE_DAT to disable power down to the chip				
				LDR     R0, =0x50000228
				MOVS   	R1,#0xC0 
				MOVS 	R2,R0
				ANDS	R2,R2,R1
				CMP 	R2, #0
				BNE		MAINLOOP
				LDR     R1, =0x01
				STR		R1, [R0]
MAINLOOP				
                LDR     R0, =__main
                BX      R0
                ENDP
                
                
; Dummy Exception Handlers (infinite loops which can be modified)                
                
NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler
				LDR		R0, [R13, #24]	; Get previous PC
				LDRH	R1, [R0]		; Get instruction
				LDR		R2, =0xBEAB		; The sepcial BKPT instruction
				CMP		R1, R2			; Test if the instruction at previous PC is BKPT
				BNE		HardFault_Handler_Ret	; Not BKPT

				ADDS	R0, #2			; Skip BKPT
				STR		R0, [R13, #24]	; Save previous PC

				LDR		R0, [R13, #28]	; Get previous xPSR
				MOVS	R1, #1
				LSLS	R1, R1, #30		; R1 = 0x40000000UL
				BICS	R0, R0, R1		; Clear flag Z
				STR		R0, [R13, #28]	; Save previous PC

				BX		LR
HardFault_Handler_Ret
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  BOD_IRQHandler            [WEAK]
                EXPORT  WDT_IRQHandler            [WEAK]
                EXPORT  EINT0_IRQHandler          [WEAK]
                EXPORT  EINT1_IRQHandler          [WEAK]
                EXPORT  GPAB_IRQHandler           [WEAK]
                EXPORT  ALC_IRQHandler            [WEAK]
                EXPORT  PWMA_IRQHandler           [WEAK]
                EXPORT  TMR0_IRQHandler           [WEAK]
                EXPORT  TMR1_IRQHandler           [WEAK]
                EXPORT  UART02_IRQHandler         [WEAK]
                EXPORT  SPI0_IRQHandler           [WEAK]
                EXPORT  I2C0_IRQHandler           [WEAK]
                EXPORT  TALARM_IRQHandler         [WEAK]
                EXPORT  ACMP_IRQHandler           [WEAK]
                EXPORT  PDMA_IRQHandler           [WEAK]
				EXPORT  I2S_IRQHandler            [WEAK]
                EXPORT  CAPS_IRQHandler           [WEAK]
                EXPORT  SDADC_IRQHandler          [WEAK]
                EXPORT  RTC_IRQHandler            [WEAK]
                
BOD_IRQHandler
WDT_IRQHandler
EINT0_IRQHandler
EINT1_IRQHandler
GPAB_IRQHandler
ALC_IRQHandler
PWMA_IRQHandler
TMR0_IRQHandler
TMR1_IRQHandler
UART02_IRQHandler
SPI0_IRQHandler
I2C0_IRQHandler
TALARM_IRQHandler
ACMP_IRQHandler
PDMA_IRQHandler
I2S_IRQHandler
CAPS_IRQHandler
SDADC_IRQHandler
RTC_IRQHandler
                B       .
                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, = (Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


				AREA    |.semihost|, CODE, READONLY
                
; int SH_DoCommand(int n32In_R0, int n32In_R1, int *pn32Out_R0);
; Input
;	R0,n32In_R0: semihost register 0
;	R1,n32In_R1: semihost register 1
; Output
;	R2,*pn32Out_R0: semihost register 0
; Return
;	0: No ICE debug
;	1: ICE debug
SH_DoCommand	PROC
				EXPORT	SH_DoCommand			[WEAK]

				BKPT	0xAB		; Wait ICE or HardFault
									; ICE will step over BKPT directly
									; HardFault will step BKPT and the next line

				B		SH_ICE
SH_HardFault						; Captured by HardFault
				MOVS	R0, #0		; Set return value to 0
				BX	 	lr			; Return

SH_ICE								; Captured by ICE
				; Save return value
				CMP		R2, #0
				BEQ		SH_End
				STR		R0, [R2]	; Save the return value to *pn32Out_R0
SH_End
				MOVS	R0, #1		; Set return value to 1
				BX	 	lr			; Return
				ENDP

                END

